@@ -1,7 +1,7 @@
 ;;; nnshimbun.el --- interfacing with web newspapers
 
-;; Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
-;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
+;; 2010, 2011 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;          ARISAWA Akihiro    <ari@mbf.sphere.ne.jp>,
@@ -267,7 +267,7 @@ shimbun articles.
 
 ;; The back end definitions:
 
-(gnus-declare-backend "nnshimbun" 'address)
+(gnus-declare-backend "nnshimbun" 'address 'server-marks)
 (nnoo-declare nnshimbun)
 
 (defvoo nnshimbun-directory (nnheader-concat gnus-directory "shimbun/")
@@ -502,7 +502,7 @@ If FULL-NAME-P is non-nil, it assumes that GROUP is a full name."
       (nnheader-report 'nnshimbun "Couldn't retrieve article: %s"
 		       (prin1-to-string article)))))
 
-(deffoo nnshimbun-request-group (group &optional server dont-check)
+(deffoo nnshimbun-request-group (group &optional server dont-check info)
   (setq group (nnshimbun-decode-group-name group))
   (if (not (nnshimbun-possibly-change-group group server))
       (nnheader-report 'nnshimbun "Invalid group")
@@ -522,12 +522,19 @@ If FULL-NAME-P is non-nil, it assumes that GROUP is a full name."
 		       lines (or beg 0) (or end 0) group))))
 
 (deffoo nnshimbun-request-scan (&optional group server)
-  (setq group (nnshimbun-decode-group-name group))
+  (when (and group
+	     (progn
+	       (setq group (nnshimbun-decode-group-name group))
+	       (nnshimbun-possibly-change-group nil server)))
+    (nnshimbun-generate-nov-database group)))
+
+(deffoo nnshimbun-retrieve-groups (groups &optional server)
   (when (nnshimbun-possibly-change-group nil server)
-    (if group
-	(nnshimbun-generate-nov-database group)
-      (dolist (group (shimbun-groups nnshimbun-shimbun))
-	(nnshimbun-generate-nov-database group)))))
+    (dolist (group groups)
+      (nnshimbun-request-scan group server)
+      (nnshimbun-request-group group server))
+    (nnshimbun-request-list server)
+    'active))
 
 (deffoo nnshimbun-close-group (group &optional server)
   (setq group (nnshimbun-decode-group-name group))
@@ -854,25 +861,34 @@ Other files in the directory are also deleted."
 	(kill-buffer nov)))
     t))
 
+(eval-when-compile
+  (unless (fboundp 'nnheader-update-marks-actions)
+    (defalias 'nnheader-update-marks-actions 'ignore)))
+
 (deffoo nnshimbun-request-set-mark (group actions &optional server)
   (setq group (nnshimbun-decode-group-name group))
   (when (and (not nnshimbun-marks-is-evil)
 	     (nnshimbun-possibly-change-group group server))
     (nnshimbun-open-marks group server)
-    (dolist (action actions)
-      (let ((range (nth 0 action))
-	    (what  (nth 1 action))
-	    (marks (nth 2 action)))
-	(assert (or (eq what 'add) (eq what 'del)) nil
-		"Unknown request-set-mark action: %s" what)
-	(dolist (mark marks)
-	  (setq nnshimbun-marks
-		(gnus-update-alist-soft
-		 mark
-		 (funcall (if (eq what 'add) 'gnus-range-add
-			    'gnus-remove-from-range)
-			  (cdr (assoc mark nnshimbun-marks)) range)
-		 nnshimbun-marks)))))
+    (if (and (fboundp 'nnheader-update-marks-actions)
+	     (not (eq (symbol-function 'nnheader-update-marks-actions)
+		      'ignore)))
+	(setq nnshimbun-marks
+	      (nnheader-update-marks-actions nnshimbun-marks actions))
+      (dolist (action actions)
+	(let ((range (nth 0 action))
+	      (what  (nth 1 action))
+	      (marks (nth 2 action)))
+	  (assert (or (eq what 'add) (eq what 'del)) nil
+		  "Unknown request-set-mark action: %s" what)
+	  (dolist (mark marks)
+	    (setq nnshimbun-marks
+		  (gnus-update-alist-soft
+		   mark
+		   (funcall (if (eq what 'add) 'gnus-range-add
+			      'gnus-remove-from-range)
+			    (cdr (assoc mark nnshimbun-marks)) range)
+		   nnshimbun-marks))))))
     (nnshimbun-save-marks group))
   nil)
 
