@@ -1,6 +1,6 @@
 ;;; sb-tech-on.el --- shimbun backend for Tech-On! -*- coding: iso-2022-7bit -*-
 
-;; Copyright (C) 2007, 2009 Katsumi Yamaoka
+;; Copyright (C) 2007, 2009, 2010 Katsumi Yamaoka
 
 ;; Author: Katsumi Yamaoka <yamaoka@jpl.org>
 ;; Keywords: news
@@ -211,11 +211,8 @@ Face: iVBORw0KGgoAAAANSUhEUgAAACAAAAAgAgMAAAAOFJJnAAAADFBMVEUAAAB/gP+ttr7///8
       (while (re-search-forward "<p>\\([\t\n ]*<p>\\)+" nil t)
 	(delete-region (match-beginning 1) (match-end 0)))
       ;; Remove useless tags.
-      (goto-char (point-min))
-      (while (and (re-search-forward "\
-<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"bpimage_click\"" nil t)
-		  (shimbun-end-of-tag "div" t))
-	(replace-match "\n"))
+      (shimbun-remove-tags
+       "\\(div\\)[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"bpimage_click\"" t)
       (when author
 	(goto-char (point-min))
 	(insert "<p>" author "</p>\n"))
