@@ -1,7 +1,6 @@
 ;;; shimbun.el --- interfacing with web newspapers -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-;; Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2001-2011 Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;         Akihiro Arisawa    <ari@mbf.sphere.ne.jp>,
@@ -79,6 +78,7 @@
 (require 'luna)
 (require 'std11)
 (require 'w3m)
+(require 'xml)
 
 (eval-and-compile
   (luna-define-class shimbun ()
@@ -318,9 +318,11 @@ slot of SHIMBUN to encode URL."
   "Return a real URL."
   (w3m-real-url url no-cache))
 
+(defalias 'shimbun-beginning-of-tag 'w3m-beginning-of-tag)
 (defalias 'shimbun-decode-anchor-string 'w3m-decode-anchor-string)
 (defalias 'shimbun-decode-entities 'w3m-decode-entities)
 (defalias 'shimbun-decode-entities-string 'w3m-decode-entities-string)
+(defalias 'shimbun-end-of-tag 'w3m-end-of-tag)
 (defalias 'shimbun-expand-url 'w3m-expand-url)
 (defalias 'shimbun-find-coding-system 'w3m-find-coding-system)
 (defalias 'shimbun-replace-in-string 'w3m-replace-in-string)
@@ -1128,9 +1130,15 @@ HEADER is a header structure obtained via `shimbun-headers'.")
 
 (defun shimbun-insert-footer (shimbun header &optional html &rest args)
   "Insert the footer and ARGS."
+  ;; Remove <p> and </p> from the last paragraph in order to prevent
+  ;; an empty line from being inserted at the end.
+  (goto-char (point-min))
+  (when (re-search-forward "[\t\n ]*<p>[\t\n ]*\\([^<]+\\)</p>[\t\n ]*\
+\\(?:<\\(?:![^>]+\\|/?div\\|/?p\\)>[\t\n ]*\\)*\\'" nil t)
+    (replace-match "\n\\1"))
   (goto-char (point-min))
   (when (re-search-forward
-	 "[\t\n ]*\\(?:<\\(?:![^>]+\\|br\\|/?div\\|/?p\\)>[\t\n ]*\\)*\\'"
+	 "[\t\n ]*\\(?:<\\(?:![^>]+\\|br\\|div\\)>[\t\n ]*\\)*\\'"
 	 nil 'move)
     (delete-region (match-beginning 0) (point-max)))
   (apply 'insert "\n" (shimbun-footer shimbun header html) args))
@@ -1150,7 +1158,8 @@ HEADER is a header structure obtained via `shimbun-headers'.")
 
 (defun shimbun-make-html-contents (shimbun header)
   (let ((base-url (or (shimbun-current-base-url)
-		      (shimbun-article-url shimbun header))))
+		      (file-name-directory
+		       (shimbun-article-url shimbun header)))))
     (when (shimbun-clear-contents shimbun header)
       (goto-char (point-min))
       (insert "<html>\n<head>\n<base href=\""
@@ -1270,7 +1279,7 @@ that the content type is text/html, otherwise text/plain."
   (if html
       (progn
 	(insert "<html>\n<head>\n<base href=\""
-		(shimbun-article-url shimbun header)
+		(file-name-directory (shimbun-article-url shimbun header))
 		"\">\n</head>\n<body>\n")
 	(shimbun-insert-footer shimbun header t "</body>\n</html>\n"))
     (shimbun-insert-footer shimbun header))
@@ -1359,110 +1368,64 @@ the following form returns the present time of Japan, wherever you are.
 		   (< (cadr a) (cadr b)))))))
 
 (defun shimbun-remove-tags (begin-tag &optional end-tag)
-  "Remove all occurrences of regions surrounded by BEGIN-TAG and END-TAG."
+  "Remove all occurrences of regions surrounded by BEGIN-TAG and END-TAG.
+
+If END-TAG is neither nil nor a string, it works strictly for open and
+close tags (i.e. <tag>...</tag>).  In that case, BEGIN-TAG should be a
+regexp matching text within an open tag \"<...>\", and buffer's text that
+matches the first non-shy sub-expression surrounded by parens \"\\\\(...\\\\)\"
+in BEGIN-TAG is regarded as a name of an open tag; if there is no paren
+in BEGIN-TAG, the whole text that matches BEGIN-TAG is regarded as a tag
+name."
   (let ((case-fold-search t))
     (goto-char (point-min))
-    (if end-tag
-	(let (pos)
-	  (while (and (re-search-forward begin-tag nil t)
-		      (setq pos (match-beginning 0))
-		      (re-search-forward end-tag nil t))
-	    (delete-region pos (point))))
-      (while (re-search-forward begin-tag nil t)
-	(delete-region (match-beginning 0) (match-end 0))))))
+    (cond ((stringp end-tag)
+	   (let (pos)
+	     (while (and (re-search-forward begin-tag nil t)
+			 (setq pos (match-beginning 0))
+			 (re-search-forward end-tag nil t))
+	       (delete-region pos (point)))))
+	  ((null end-tag)
+	   (while (re-search-forward begin-tag nil t)
+	     (delete-region (match-beginning 0) (match-end 0))))
+	  (t
+	   (setq begin-tag
+		 (if (string-match "\\\\([^?]" begin-tag)
+		     (concat "<[\t\n\r ]*" begin-tag)
+		   (concat "<[\t\n\r ]*\\(" begin-tag "\\)[\t\n\r >]")))
+	   (while (re-search-forward begin-tag nil t)
+	     (when (or (not (eq (match-end 0) (match-end 1)))
+		       (memq (char-after) '(?\t ?\n ?\r ?  ?>)))
+	       (goto-char (match-beginning 0))
+	       (if (shimbun-end-of-tag (match-string 1) t)
+		   (replace-match "\n")
+		 (goto-char (match-end 0)))))))))
 
-(defun shimbun-end-of-tag (&optional tag include-whitespace)
-  "Move point to the end of tag.  Inner nested tags are skipped.
-If TAG, which is a name of the tag, is given, this function moves point
-from the open-tag <TAG ...> (point should exist in front of or within
-it initially) to the end-point of the closing-tag </TAG>.  For example,
-in the following two situations, point moves from the leftmost tag to
-the end-point of the rightmost tag:
-
-<TAG ...>...<TAG ...>...<TAG ...>...</TAG>...</TAG>...</TAG>
-<TAG ...>...<TAG ...>...</TAG>...<TAG ...>...</TAG>...</TAG>
-
-If TAG is omitted or nil, this function moves point to the end-point of
-the tag in which point exists.  In this case, point should initially
-exist within the start position of the tag and the next tag as follows:
-
-<!-- foo <bar ...<baz ...>...> -->
- ^^^^^^^^
-If INCLUDE-WHITESPACE is non-nil, include leading and trailing
-whitespace.  Return the end-point and set the match-data #0, #1, #2,
-and #3 as follows (\"___\" shows whitespace):
-
-The case where TAG is spefified:
-___<TAG ...>___...___</TAG>___
-   0        1  2  2  1     0     INCLUDE-WHITESPACE=nil
-0  1        2  3  3  2     1  0  INCLUDE-WHITESPACE=non-nil
-
-The case where TAG is nil:
-___<TAG ...>___
-   0        0     INCLUDE-WHITESPACE=nil
-0  1        1  0  INCLUDE-WHITESPACE=non-nil"
-  (let ((init (point))
-	(num 1)
-	(md (match-data))
-	(case-fold-search t)
-	regexp st1 st2 st3 nd1 nd2 nd3 nd0 st0)
-    (condition-case nil
-	(progn
-	  (if tag
-	      (progn
-		(setq tag (regexp-quote tag))
-		(if (looking-at (concat "\
-\[\t\n\r ]*\\(<[\t\n\r ]*" tag "\\(?:[\t\n\r ]*\\|[\t\n\r ]+[^>]+\\)>\\)\
-\[\t\n\r ]*"))
-		    (setq st1 (nth 2 (match-data)) ;; (match-beginning 1)
-			  st2 (nth 3 (match-data)) ;; (match-end 1)
-			  st3 (nth 1 (match-data))) ;; (match-end 0)
-		  (search-backward "<")
-		  (if (looking-at (concat "\
-\\(<[\t\n\r ]*" tag "\\(?:[\t\n\r ]*\\|[\t\n\r ]+[^>]+\\)>\\)[\t\n\r ]*"))
-		      (setq st1 (car (match-data)) ;; (match-beginning 0)
-			    st2 (nth 3 (match-data)) ;; (match-end 1))
-			    st3 (nth 1 (match-data))) ;; (match-end 0)
-		    (error "")))
-		(goto-char (1+ st1))
-		(setq regexp (concat "\
-\[\t\n\r ]*\\(<\\(/\\)?" tag "\\(?:[\t\n\r ]*\\|[\t\n\r ]+[^>]+\\)>\\)"))
-		(while (and (> num 0)
-			    (re-search-forward regexp))
-		  (setq num (if (match-beginning 2)
-				(1- num)
-			      (1+ num))))
-		(setq nd1 (nth 3 (match-data)) ;; (match-end 1)
-		      nd2 (nth 2 (match-data)) ;; (match-beginning 1)
-		      nd3 (car (match-data)))) ;; (match-beginning 0)
-	    (search-backward "<")
-	    (setq st1 (car (match-data))) ;; (match-beginning 0)
-	    (goto-char init)
-	    (while (and (> num 0)
-			(re-search-forward "\\(>\\)\\|<"))
-	      (setq num (if (match-beginning 1)
-			    (1- num)
-			  (1+ num))))
-	    (setq nd1 (nth 3 (match-data)))) ;; (match-end 1)
-	  (if include-whitespace
-	      (progn
-		(skip-chars-forward "\t\n\r ")
-		(setq nd0 (point-marker))
-		(goto-char st1)
-		(skip-chars-backward "\t\n\r ")
-		(setq st0 (point-marker))
-		(goto-char nd0)
-		(set-match-data (if tag
-				    (list st0 nd0 st1 nd1 st2 nd2 st3 nd3)
-				  (list st0 nd0 st1 nd1))))
-	    (set-match-data (if tag
-				(list st1 nd1 st2 nd2 st3 nd3)
-			      (list st1 nd1))))
-	  (point))
-      (error
-       (set-match-data md)
-       (goto-char init)
-       nil))))
+(defun shimbun-remove-orphaned-tag-strips (string)
+  "Remove orphaned tag strips that match STRING.
+STRING is like the inside of a \"\\\\(...\\\\)\" in a regular expression."
+  (let (regexp st nd)
+    (setq regexp (concat "<[\t\n\r ]*/[\t\n\r ]*\\(" string
+			 "\\)\\(?:[\t\n\r ][^>]+\\)?[\t\n\r ]*>[\t ]*"))
+    (goto-char (point-max))
+    (while (re-search-backward regexp nil t)
+      (setq st (match-beginning 0)
+	    nd (goto-char (match-end 0)))
+      (unless (prog1
+		  (shimbun-beginning-of-tag (match-string 1))
+		(goto-char st))
+	(skip-chars-backward "\t\n\r ")
+	(delete-region (point) nd)))
+    (setq regexp (concat "[\t\n\r ]*<[\t\n\r ]*\\(" string
+			 "\\)\\(?:[\t\n\r ][^>]+\\)?[\t\n\r ]*>[\t ]*"))
+    (goto-char (point-min))
+    (while (re-search-forward regexp nil t)
+      (setq st (goto-char (match-beginning 0))
+	    nd (match-end 0))
+      (unless (prog1
+		  (shimbun-end-of-tag (match-string 1))
+		(goto-char nd))
+	(delete-region st nd)))))
 
 (defun shimbun-remove-markup ()
   "Remove all HTML markup, leaving just plain text."
@@ -1852,6 +1815,40 @@ There are exceptions; some chars aren't converted, and \"$B!c(B\", \"$B!d(B\
     (unless (eobp)
       (shimbun-japanese-hankaku-region start (point-max) quote))))
 
+(defun shimbun-xml-parse-buffer ()
+  "Calls (lib)xml-parse-region on the whole buffer.
+This is a wrapper for xml-parse-region, which will resort to
+using libxml-parse-xml-region if available, since it is much
+faster."
+  (if (fboundp 'libxml-parse-xml-region)
+      (save-excursion
+	(goto-char (point-min))
+	(let ((xml (libxml-parse-xml-region
+		    (1- (search-forward "<" nil t)) (point-max)))
+	      (start 0)
+	      (stylestring
+	       (progn (goto-char (point-min))
+		      (when (re-search-forward "<\\(rss\\|feed\\)\\(.*?\\)>" nil t)
+			(match-string 2))))
+	      stylesheet)
+	  ;; Parse the stylesheet
+	  (when stylestring
+	    (while (string-match "\\(xmlns:?.*?\\)=\"\\(.*?\\)\"" stylestring start)
+	      (setq start (match-end 0))
+	      (push (cons (intern (match-string 1 stylestring))
+			  (match-string 2 stylestring))
+		    stylesheet)))
+	  ;; Add stylesheet into XML structure
+	  (when stylesheet
+	    (if (nth 1 xml)
+		(nconc (nth 1 xml) stylesheet)
+	      (setcar (cdr xml) stylesheet)))
+	  (list xml)))
+    ;; We don't have libxml, so just use the slow one.
+    (xml-parse-region (point-min) (point-max))))
+
+
+
 (provide 'shimbun)
 
 ;;; shimbun.el ends here
